/*
 * Copyright (C) Jason Qingpei Wang (wangqingpei@gmail.com)
 * Copyright (C) Leandro Dorileo
 * Copyright (C) 2009 coresystems GmbH
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 */
.global _setup
.text
.code16

 rom_size   =     0x80      # ROM size in multiple of 512 bytes
/* Enter your PCI IDs here: */
 PCI_VENDORID =	  0x1002
 PCI_DEVICEID =   0x4387

 .word 0xAA55          # Rom signature 
 .byte rom_size        # Size of this ROM, see definition above 
 jmp _setup            # jump to PCI initialization function
 .org 0x18
 .word pci_header      # Pointer to PCI HDR structure (at 18h)
 .word pnp_header      # PnP Expansion Header Pointer (at 1Ah)

# --------------------------------------------
# PCI data structure
#

pci_header:
	.byte 'P','C','I','R'
	.word PCI_VENDORID	/* PCI Vendor ID */
	.word PCI_DEVICEID	/* PCI Device ID */
	.word 0x0000	/* Vital Product Data */
	.word pci_header_end - pci_header	/* header length */
	.byte 0x00	/* PCI header revision */
	.byte 0x00, 0x80, 0x01	/* Class Code (SCSI) */
	.word 0x40	/* This image size */
	.word 0x0000	/* rom revision */
	.byte 0x00	/* rom type: x86 bios */
	.word 0x81	/* No further images in this rom */
	.word 0x0000	/* reserved */
pci_header_end:

# --------------------------------------------
# PNP data structure
#

pnp_header:
	.byte '$','P','n','P'
	.byte 0x01			/* header revision */
	.byte 0x02			/* length */
	.word 0x0000			/* next header offset */
	.byte 0x00			/* reserved */
	.word 0x0000			/* device id */
	.word vendor_name - pnp_header	/* vendor name */
	.word device_name - pnp_header	/* device name */
	.byte 0x00, 0x00, 0x00		/* device type */
	.byte 0x00			/* device indicator */
	.word 0x00			/* boot connection vector */
	.word 0x0000			/* disconnect vector (unused) */
	.word 0x0000			/* bootstrap entry point (unused) */
	.word 0x0000			/* reserved */
	.word 0x0000			/* static resource info (unused) */

vendor_name:
	.string "coresystems GmbH"
device_name:
	.string "USB BootROM"


#--------------------------------------------------------------------
# PCI Option ROM initialization Code (init function)
#

_setup:
  movw %cs, %ax	
  movw %ax, %ds	
  cli            # disable interrupt
  xorl %eax,%eax
  movw %ds,%ax
  shll $4,%eax
  addl $gdt_marker,%eax
  movl %eax,(gdt_desc+2)
  lgdtl %cs:gdt_desc  # load GDT to GDTR (we load both limit and base address)

  mov %cs,%ax
  mov %ax,%ss
  movl %cr0, %eax     # switch to P-Mode
  or   $1, %eax
  movl %eax, %cr0     # haven't yet in P-Mode, we need a FAR Jump

  .byte 0x66, 0xea   # prefix + jmpi-opcode (this force P-Mode to be reached i.e. CS updated)
  .long (0xce000+do_pm)        # 32-bit linear address (jump target)
  .word SEG_CODE_SEL # code segment selector

.code32
do_pm:
  xorl %esi, %esi
  xorl %edi, %edi
  movw $0x10, %ax  # Save data segment identifier (see GDT)
  movw %ax, %ds
  movw $0x18, %ax  # Save stack segment identifier
1:	jmp 1b
  jmp  main # jump to main function



# -----------------------------------------------------
#			GDT definition
#
.align 8, 0 # align GDT in 8 bytes boundary
gdt_marker:   # dummy Segment Descriptor (GDT)
  .long 0
  .long 0

SEG_CODE_SEL = ( . - gdt_marker)
SegDesc1:      # kernel CS (08h) PL0, 08h is an identifier
  .word 0xffff # seg_length0_15
  .word 0x0000      # base_addr0_15
  .byte 0x00      # base_addr16_23
  .byte 0x9f   # flags
  .byte 0xcf   # access
  .byte 0      # base_addr24_31

SEG_DATA_SEL = ( . - gdt_marker)
SegDesc2:      # kernel DS (10h) PL0
  .word	0xffff # seg_length0_15
  .word	0      # base_addr0_15
  .byte 0      # base_addr16_23
  .byte 0x93   # flags
  .byte 0xcf   # access
  .byte 0      # base_addr24_31

SEG_STACK_SEL = ( . - gdt_marker)
SegDesc3:      # kernel SS (18h) PL0
  .word 0xffff # seg_length0_15
  .word 0      # base_addr0_15
  .byte 0      # base_addr16_23
  .byte 0x9f   # flags
  .byte 0xcf   # access
  .byte 0      # base_addr24_31


  .word 0xffff # seg_length0_15
  .word 0      # base_addr0_15
  .byte 0      # base_addr16_23
  .byte 0x93   # flags
  .byte 0xcf   # access
  .byte 0      # base_addr24_31

  .word 0xffff # seg_length0_15
  .word 0      # base_addr0_15
  .byte 0      # base_addr16_23
  .byte 0x9f   # flags
  .byte 0xcf   # access
  .byte 0      # base_addr24_31

  .word 0xffff # seg_length0_15
  .word 0      # base_addr0_15
  .byte 0      # base_addr16_23
  .byte 0x93   # flags
  .byte 0xcf   # access
  .byte 0      # base_addr24_31

  .word 0xffff # seg_length0_15
  .word 0      # base_addr0_15
  .byte 0      # base_addr16_23
  .byte 0x9b   # flags
  .byte 0x8f   # access
  .byte 0      # base_addr24_31

  .word 0xffff # seg_length0_15
  .word 0      # base_addr0_15
  .byte 0      # base_addr16_23
  .byte 0x93   # flags
  .byte 0x8f   # access
  .byte 0      # base_addr24_31

gdt_end:

gdt_desc:
	.word (gdt_end - gdt_marker - 1) # GDT limit
        .long gdt_marker # physical addr of GDT
